/* 
 * Copyright (C) 2006 Piotr Pokora <piotr.pokora@infoglob.pl>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _MIDGARD_BLOB_H
#define _MIDGARD_BLOB_H

/**
 * \defgroup midgard_blob Midgard Blob
 * 
 * Simple class which is intented to manage file descriptors via GIOChannel interface.
 * MidgardBlob encapsulate relations among all typical MidgardSchema objects, midgard_attachment
 * objects and MidgardConnection handler. It's designed as a helper to build language bindings
 * written for Midgard system. 
 */


#include "midgard/midgard_connection.h"
#include "midgard/types.h"

G_BEGIN_DECLS

/* convention macros */
#define MIDGARD_TYPE_BLOB (midgard_blob_get_type())
#define MIDGARD_BLOB(object)  \
	(G_TYPE_CHECK_INSTANCE_CAST ((object),MIDGARD_TYPE_BLOB, MidgardBlob))
#define MIDGARD_BLOB_CLASS(klass)  \
	(G_TYPE_CHECK_CLASS_CAST ((klass), MIDGARD_TYPE_BLOB, MidgardBlobClass))
#define MIDGARD_IS_BLOB(object)   \
	(G_TYPE_CHECK_INSTANCE_TYPE ((object), MIDGARD_TYPE_BLOB))
#define MIDGARD_IS_BLOB_CLASS(klass) \
	(G_TYPE_CHECK_CLASS_TYPE ((klass), MIDGARD_TYPE_BLOB))
#define MIDGARD_BLOB_GET_CLASS(obj) \
	(G_TYPE_INSTANCE_GET_CLASS ((obj), MIDGARD_TYPE_BLOB, MidgardBlobClass))

typedef struct MidgardBlob MidgardBlob;
typedef struct MidgardBlobClass MidgardBlobClass;
typedef struct _MidgardBlobPrivate MidgardBlobPrivate;

/**
 * \ingroup midgard_blob
 *
 * MidgardObjectClass structure.
 */ 
struct MidgardBlobClass {
	GObjectClass parent;

	GIOChannel* (*read) (MidgardBlob *self);
	GIOChannel* (*write) (MidgardBlob *self, const gchar *content);
	void (*destroy) (MidgardBlob *self);
};

/**
 * \ingroup midgard_blob
 *
 * MidgardObject structure
 */ 
struct MidgardBlob {
	GObject parent;

	MidgardBlobPrivate *private;
};

/**
 * \ingroup midgard_blob
 *
 * Returns Midgard Blob Gtype value.
 * Registers this type unless already registered.
 */ 
extern GType 
midgard_blob_get_type(void);

/**
 * \ingroup midgard_blob
 * 
 * Instatiate new Midgard Blob.
 *
 * \param mgd , MidgardConection pointer 
 * \param attachment , MidgardObject of MIDGARD_TYPE_ATTACHMENT type.
 * 
 * \return MidgardBlob, newly instatiated MidgardBlob object.
 * 
 * Instatiate new Midgard Blob object for the given midgard_attachment object. 
 * This is almost the same constructor as g_object_new, but unlike that one,
 * midgard_object_new requires Midgard Connection handler's pointer.
 * Initializes new Midgard Blob object and its internal data in currently 
 * used MidgardConnection context.  
 */
extern MidgardBlob *
midgard_blob_new(MidgardConnection *mgd, MgdObject *attachment);

/**
 * \ingroup midgard_blob
 *
 * Returns GIOChannel pointer for the given midgard_blob.
 *
 * \param self, Midgard Blob self instance
 * 
 * \return GIOChannel which holds filename's content, NULL if there is no filename
 * associated with midgard_attachment for the given Midgard Blob instance.
 *
 * Returned pointer is owned by midgard-core and should not be freed.
 *
 */ 
extern GIOChannel *
midgard_blob_read(MidgardBlob *self);

/**
 * \ingroup midgard_blob
 *
 * Returns GIOChannel pointer for written blob file.
 *
 * \param self, MidgardBlob self instance.
 * \param content, content which should be written to file.
 *
 * \return GIOChannel pointer if content has been successfully written to file.
 * Otherwise returns NULL.
 */ 
extern GIOChannel *
midgard_blob_write(MidgardBlob *self,
				const gchar *content);

/**
 * \ingroup midgard_blob
 * 
 * Destroy Midgard Blob instance.
 *
 * \param self, MidgardBlob self instance.
 *
 * Destroy Midgard Blob instance and all data associated with this instance.
 * Any file handler assigned to destroyed instance will be freed as well.
 * It's enough to call g_object_unref to destroy Midgard Blob. This method is hidden 
 * internally in MidgardObjectClass finalize method and it's added for readability.
 */ 
extern void 
midgard_blob_destroy(MidgardBlob *self);

G_END_DECLS

#endif /* _MIDGARD_BLOB_H */
